@@ -5,7 +5,7 @@ from types import ModuleType
 import Namcap.ruleclass
 
 # Tarball rules
-from . import (  # noqa: ignore=F401
+from . import (  # noqa: F401
     anyelf,
     elffiles,
     emptydir,
@@ -28,6 +28,7 @@ from . import (  # noqa: ignore=F401
     permissions,
     py_mtime,
     pydepends,
+    qmldepends,
     rpath,
     runpath,
     scrollkeeper,
@@ -39,7 +40,7 @@ from . import (  # noqa: ignore=F401
 )
 
 # PKGBUILD and metadata rules
-from . import (  # noqa: ignore=F401
+from . import (  # noqa: F401
     arrays,
     badbackups,
     carch,
